@@ -96,7 +96,8 @@ get_secrets_from_user (const char *request_id,
 		char *pwd = NULL;
 
 		/* Ask user for the password */
-		g_print ("%s\n", msg);
+		if (msg)
+			g_print ("%s\n", msg);
 		if (secret->value) {
 			/* Prefill the password if we have it. */
 			rl_startup_hook = set_deftext;
@@ -148,7 +149,10 @@ do_agent_secret (NmCli *nmc, int argc, char **argv)
 		nmc->should_wait++;
 
 		nm_secret_agent_simple_enable (NM_SECRET_AGENT_SIMPLE (nmc->secret_agent), NULL);
-		g_signal_connect (nmc->secret_agent, "request-secrets", G_CALLBACK (secrets_requested), nmc);
+		g_signal_connect (nmc->secret_agent,
+		                  NM_SECRET_AGENT_SIMPLE_REQUEST_SECRETS,
+		                  G_CALLBACK (secrets_requested),
+		                  nmc);
 		g_print (_("nmcli successfully registered as a NetworkManager's secret agent.\n"));
 	} else {
 		g_string_printf (nmc->return_text, _("Error: secret agent initialization failed"));
@@ -205,25 +209,16 @@ do_agent_all (NmCli *nmc, int argc, char **argv)
 }
 
 static const NMCCommand agent_cmds[] = {
-	{"secret",  do_agent_secret,  usage_agent_secret },
-	{"polkit",  do_agent_polkit,  usage_agent_polkit },
-	{"all",     do_agent_all,     usage_agent_all },
-	{NULL,      do_agent_all,     usage }
+	{ "secret",  do_agent_secret,  usage_agent_secret,  TRUE,   TRUE },
+	{ "polkit",  do_agent_polkit,  usage_agent_polkit,  TRUE,   TRUE },
+	{ "all",     do_agent_all,     usage_agent_all,     TRUE,   TRUE },
+	{ NULL,      do_agent_all,     usage,               TRUE,   TRUE },
 };
 
-
 NMCResultCode
 do_agent (NmCli *nmc, int argc, char **argv)
 {
-	/* Get NMClient object */
-	nmc->get_client (nmc);
+	nmc_do_cmd (nmc, agent_cmds, *argv, argc, argv);
 
-	/* Check whether NetworkManager is running */
-	if (!nm_client_get_nm_running (nmc->client)) {
-		g_string_printf (nmc->return_text, _("Error: NetworkManager is not running."));
-		nmc->return_value = NMC_RESULT_ERROR_NM_NOT_RUNNING;
-		return nmc->return_value;
-	}
-
-	return nmc_do_cmd (nmc, agent_cmds, *argv, argc, argv);
+	return nmc->return_value;
 }
