@@ -121,14 +121,14 @@ struct _NMPCacheId {
 			char ifname_short[IFNAMSIZ - 1]; /* don't include the trailing NUL so the struct fits in 4 bytes. */
 		} link_by_ifname;
 		struct _nm_packed {
-			/* NMP_CACHE_ID_TYPE_ROUTES_BY_DESTINATION_IP6 */
+			/* NMP_CACHE_ID_TYPE_ROUTES_BY_DESTINATION_IP4 */
 			guint8 _id_type;
 			guint8 plen;
 			guint32 _misaligned_metric;
 			guint32 _misaligned_network;
 		} routes_by_destination_ip4;
 		struct _nm_packed {
-			/* NMP_CACHE_ID_TYPE_ROUTES_BY_DESTINATION_IP4 */
+			/* NMP_CACHE_ID_TYPE_ROUTES_BY_DESTINATION_IP6 */
 			guint8 _id_type;
 			guint8 plen;
 			guint32 _misaligned_metric;
@@ -206,6 +206,10 @@ typedef struct {
 	NMPlatformLnkIpIp _public;
 } NMPObjectLnkIpIp;
 
+typedef struct {
+	NMPlatformLnkMacsec _public;
+} NMPObjectLnkMacsec;
+
 typedef struct {
 	NMPlatformLnkMacvlan _public;
 } NMPObjectLnkMacvlan;
@@ -267,6 +271,9 @@ struct _NMPObject {
 		NMPlatformLnkIp6Tnl     lnk_ip6tnl;
 		NMPObjectLnkIp6Tnl      _lnk_ip6tnl;
 
+		NMPlatformLnkMacsec     lnk_macsec;
+		NMPObjectLnkMacsec      _lnk_macsec;
+
 		NMPlatformLnkMacvlan    lnk_macvlan;
 		NMPObjectLnkMacvlan     _lnk_macvlan;
 
@@ -400,7 +407,6 @@ guint nmp_cache_id_hash (const NMPCacheId *id);
 NMPCacheId *nmp_cache_id_clone (const NMPCacheId *id);
 void nmp_cache_id_destroy (NMPCacheId *id);
 
-NMPCacheId *nmp_cache_id_copy (NMPCacheId *id, const NMPCacheId *src);
 NMPCacheId *nmp_cache_id_init_object_type (NMPCacheId *id, NMPObjectType obj_type, gboolean visible_only);
 NMPCacheId *nmp_cache_id_init_addrroute_visible_by_ifindex (NMPCacheId *id, NMPObjectType obj_type, int ifindex);
 NMPCacheId *nmp_cache_id_init_routes_visible (NMPCacheId *id, NMPObjectType obj_type, gboolean with_default, gboolean with_non_default, int ifindex);
